import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.offsetbox import OffsetImage, AnnotationBbox
import matplotlib.ticker as ticker

import scienceplots
plt.style.use(['science', 'ieee', 'no-latex'])
plt.rcParams.update({'figure.dpi': 300})


# Original data and printing method data
original = {
    'Measurement': ['CE length', 'Largest width', 'Smallest width', 'Hexagonal width', 'KE length'],
    'Value': [11.00, 10.60, 6.30, 4.70, 2.50]
}

# TODO this should use 5 points in the future and calculate the mean for the plot, or this should already be the mean value!
methods = {
    'Method': ['SLS', 'FDM', 'SLA'],
    'Height (conveying element)': [11.11, 11.11, 11.04],
    'Largest Width (Ellipse)': [10.56, 10.65, 10.59],
    'Smallest Width (Ellipse)': [6.24, 6.41, 6.22],
    'Inner Width Hexagonal (Flat Surface to Flat Surface)': [4.52, 4.25, 4.60],
    'Height (Single Kneading Block)': [2.67, 2.61, 2.66],
    'Std': [0.1587, 0.1966, 0.0924],
    'Mean': [-0.036, -0.052, -0.056],
	'Error': [[0.028, 0.046, 0.026, 0.028, 0.020], [0.018,0.050,0.064,0.038,0.037], [0.032,0.020,0.015,0.015,0.039]]
}

# Create DataFrames
df_original = pd.DataFrame(original)
df_methods = pd.DataFrame(methods)

# Calculate deviations from the original data
deviations = {
    'Measurement': original['Measurement'],
    'SLS': [(methods['Height (conveying element)'][0] - original['Value'][0])/original['Value'][0], (methods['Largest Width (Ellipse)'][0] - original['Value'][1])/original['Value'][1], (methods['Smallest Width (Ellipse)'][0] - original['Value'][2])/original['Value'][2], (methods['Inner Width Hexagonal (Flat Surface to Flat Surface)'][0] - original['Value'][3])/original['Value'][3], (methods['Height (Single Kneading Block)'][0] - original['Value'][4])/original['Value'][4]],
    'FDM': [(methods['Height (conveying element)'][1] - original['Value'][0])/original['Value'][0], (methods['Largest Width (Ellipse)'][1] - original['Value'][1])/original['Value'][1], (methods['Smallest Width (Ellipse)'][1] - original['Value'][2])/original['Value'][2], (methods['Inner Width Hexagonal (Flat Surface to Flat Surface)'][1] - original['Value'][3])/original['Value'][3], (methods['Height (Single Kneading Block)'][1] - original['Value'][4])/original['Value'][4]],
    'SLA': [(methods['Height (conveying element)'][2] - original['Value'][0])/original['Value'][0], (methods['Largest Width (Ellipse)'][2] - original['Value'][1])/original['Value'][1], (methods['Smallest Width (Ellipse)'][2] - original['Value'][2])/original['Value'][2], (methods['Inner Width Hexagonal (Flat Surface to Flat Surface)'][2] - original['Value'][3])/original['Value'][3], (methods['Height (Single Kneading Block)'][2] - original['Value'][4])/original['Value'][4]]
}

# Standard deviations for each measurement
std_devs = {
    'SLS': np.array(methods['Error'][0])/np.array([methods['Height (conveying element)'][0],methods['Largest Width (Ellipse)'][0],methods['Smallest Width (Ellipse)'][0],methods['Inner Width Hexagonal (Flat Surface to Flat Surface)'][0],methods['Height (Single Kneading Block)'][0]]),
    'FDM': np.array(methods['Error'][1])/np.array([methods['Height (conveying element)'][1],methods['Largest Width (Ellipse)'][1],methods['Smallest Width (Ellipse)'][1],methods['Inner Width Hexagonal (Flat Surface to Flat Surface)'][1],methods['Height (Single Kneading Block)'][1]]),
    'SLA': np.array(methods['Error'][2])/np.array([methods['Height (conveying element)'][2],methods['Largest Width (Ellipse)'][2],methods['Smallest Width (Ellipse)'][2],methods['Inner Width Hexagonal (Flat Surface to Flat Surface)'][2],methods['Height (Single Kneading Block)'][2]])
}

# Create a DataFrame for deviations
df_deviations = pd.DataFrame(deviations)
df_std_devs = pd.DataFrame(std_devs)

# Set the figure size
plt.figure(figsize=(6, 3))

# Number of measurements
n_measurements = len(df_deviations['Measurement'])
index = np.arange(n_measurements)

# Bar width for distances
bar_width = 0.25

# Plot bars for each method with error bars for standard deviation
plt.bar(index - bar_width, df_deviations['SLS'], bar_width, yerr=df_std_devs['SLS'][:], capsize=5, label='SLS', ecolor='teal')
plt.bar(index, df_deviations['FDM'], bar_width, yerr=df_std_devs['FDM'][:], capsize=5, label='FDM', ecolor='teal')
plt.bar(index + bar_width, df_deviations['SLA'], bar_width, yerr=df_std_devs['SLA'][:], capsize=5, label='SLA', ecolor='teal')


plt.ylabel('$\Delta$ [-]')
plt.xticks(index, df_deviations['Measurement'], rotation=45, ha='right')
plt.legend()

# suppress minor ticks on x-axis
plt.gca().xaxis.set_minor_locator(ticker.NullLocator())

# Add a vertical line at x=0
plt.axhline(y=0, color='black', linewidth=0.5)

# Load and add images to the x-axis labels
image_paths = [
    'CELength.eps',
    'EllipseWidthLarge.eps',
    'EllipseWidthSmall.eps',
    'HexagonalWidth.eps',
    'KELength.eps'
]

# Specify box_alignment values for each image
box_alignments = [
    (0.1, 4.6),  # Alignment for height_conveying_element.png
    (0.1, 4.3),  # Alignment for largest_width_ellipse.png
    (0.1, 4.3),  # Alignment for smallest_width_ellipse.png
    (0.1, 4.3),  # Alignment for inner_width_hexagonal.png
    (0.1, 10.7)   # Alignment for height_single_kneading_block.png
]

# Specify zoom for each image
zooms = [
	0.035,
	0.05,
	0.05,
	0.05,
	0.07,
]

# Create the images and add them to the plot
for i, image_path in enumerate(image_paths):
    img = plt.imread(image_path)
    imagebox = OffsetImage(img, zoom=zooms[i])
    ab = AnnotationBbox(imagebox, (index[i], 0), frameon=False, box_alignment=box_alignments[i])
    plt.gca().add_artist(ab)
	
plt.tight_layout()
plt.savefig("Accuracy.eps")
plt.show()